﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;

namespace Smdn.Formats.Ini {
  public class Document : IEnumerable<Section> {
    public Section this[string name] {
      get
      {
        if (!sections.ContainsKey(name)) {
          AppendSection(name);
        }

        return sections[name];
      }
    }

    public Section[] Sections
    {
      get { return (new List<Section>(sections.Values)).ToArray(); }
    }

    public Document()
    {
      sections = new Dictionary<string, Section>();

      // append default section
      AppendSection(string.Empty);
    }

    public static Document Load(string path)
    {
      using (var reader = new StreamReader(path)) {
        return Load(reader);
      }
    }

    public static Document Load(TextReader reader)
    {
      return Parser.Parse(reader);
    }

    public void Save(string path)
    {
      using (var writer = new StreamWriter(path)) {
        Save(writer);
      }
    }

    public void Save(TextWriter writer)
    {
      Formatter.Format(this, writer);
    }

    IEnumerator IEnumerable.GetEnumerator()
    {
      return GetEnumerator();
    }

    public IEnumerator<Section> GetEnumerator()
    {
      return sections.Values.GetEnumerator();
    }

    private void AppendSection(string name)
    {
      var section = new Section(name);

      sections.Add(section.Name, section);
    }

    public override string ToString()
    {
      return string.Format("{{Sections={0}}}", sections.Count);
    }

    private Dictionary<string, Section> sections;
  }
}
