using System;

namespace Smdn.Protocols.Imap4 {
  internal static class ImapConverter
  {
    public static ImapAuthenticationType ToAuthType(string authType)
    {
      switch (authType.ToUpper()) {
        case "LOGIN":
          return ImapAuthenticationType.Login;
        case "PLAIN":
          return ImapAuthenticationType.Plain;
        case "CRAM-MD5":
          return ImapAuthenticationType.CramMd5;
        case "KERBEROS_V4":
          return ImapAuthenticationType.KerberosV4;
        case "*":
          return ImapAuthenticationType.Any;
        default:
          throw new NotSupportedException("unsupported authentication type");
      }
    }

    public static string ToString(ImapAuthenticationType authType)
    {
      switch (authType) {
        case ImapAuthenticationType.Login:
          return "login";
        case ImapAuthenticationType.Plain:
          return "plain";
        case ImapAuthenticationType.CramMd5:
          return "cram-md5";
        case ImapAuthenticationType.KerberosV4:
          return "kerberos_v4";
        case ImapAuthenticationType.Any:
          return "*";
        case ImapAuthenticationType.None:
          return string.Empty;
        default:
          throw new ArgumentOutOfRangeException();
      }
    }

    public static ImapListType ToListType(string listType)
    {
      switch (listType.ToUpper()) {
        case "LIST":
          return ImapListType.List;
        case "LSUB":
          return ImapListType.Lsub;
        default:
          throw new FormatException("invalid list type, it must be LIST or LSUB");
      }
    }

    public static string ToString(ImapListType listType)
    {
      switch (listType) {
        case ImapListType.List:
          return "list";
        case ImapListType.Lsub:
          return "lsub";
        default:
          throw new ArgumentOutOfRangeException();
      }
    }
  }
}
