using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class CheckTransaction : TransactionBase {

    // ProcessCheck
    //   ->WaitForCheck
    // WaitForCheck
    //   x Conclude

    public CheckTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessCheck;
    }

    private void ProcessCheck()
    {
      lastTag = Send("check");

      Transit(WaitForCheck);
    }

    private void WaitForCheck()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != lastTag)
          return;

        if (status.Status == ImapProtocolStatus.OK) {
          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "check completed"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string lastTag;
  }
}