using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class GreetingTransaction : TransactionBase {

    // WaitForGreeting
    //   x Conclude

    public GreetingTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      Trace.Verbose("[IMAP4rev1] waiting for server greeting response");

      return WaitForGreeting;
    }

    private void WaitForGreeting()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.ResponseCode != null) {
          ImapCapability capability;

          if (ImapStatusResponseConverter.FromCapability(status, out capability)) {
            if ((capability.KnownCapabilities & ImapKnownCapabilities.Imap4Rev1) == 0) {
              Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "server doesn't support IMAP4rev1"));
              return;
            }

            Connection.ServerInfo.Capability = capability;
          }
        }

        if (status.Status == ImapProtocolStatus.OK) {
          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "greeted"));
        }
        else if (status.Status == ImapProtocolStatus.PreAuth) {
          Conclude(new ImapResponse(ImapTransactionStatus.PreAuthed, "preauthed"));
        }
        else {
          ConcludeError(status);
        }
      }
    }
  }
}
