using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class LoginTransaction : TransactionBase {

    // ProcessParameterNotSetted
    //   x Conclude
    // ProcessLogin
    //   ->WaitForLogin
    // WaitForLogin
    //   x Conclude

    public LoginTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Parameters.ContainsKey("Username"))
        return ProcessParameterNotSetted;

      username = Request.Parameters["Username"];

      if (Request.Parameters.ContainsKey("Password"))
        password = Request.Parameters["Password"];
      else
        password = string.Empty;

      return ProcessLogin;
    }

    private void ProcessParameterNotSetted()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "paramater 'Username' must be setted"));
    }

    private void ProcessLogin()
    {
      lastTag = Send("login", username, password);

      Transit(WaitForLogin);
    }

    private void WaitForLogin()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != lastTag)
          return;

        if (status.Status == ImapProtocolStatus.OK) {
          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "logged in"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string lastTag = string.Empty;
    private string username = string.Empty;
    private string password = string.Empty;
  }
}
