using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class XyzzyTransaction : TransactionBase {

    // ProcessXyzzyIncapable
    //   x Conclude
    // ProcessXyzzy
    //   ->WaitForXyzzy
    //   x Conclude
    // WaitForXyzzy
    //   x Conclude

    public XyzzyTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if ((Connection.ServerInfo.Capability.KnownCapabilities & ImapKnownCapabilities.Xyzzy) != 0)
        return ProcessXyzzy;
      else
        return ProcessXyzzyIncapable;
    }

    private void ProcessXyzzyIncapable()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.Incapable, "XYZZY incapable"));
    }

    private void ProcessXyzzy()
    {
      lastTag = Send("XYZZY");

      Transit(WaitForXyzzy);
    }

    private void WaitForXyzzy()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != lastTag)
          return;

        if (status.Status == ImapProtocolStatus.OK) {
          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "xyzzy!"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string lastTag = string.Empty;
  }
}
