using System;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Transaction.Transactions {
  internal class DeleTransaction : TransactionBase {

    // ProcessParameterNotSetted
    //   x Conclude
    // ProcessDele
    //   ->WaitForDele
    // WaitForDele
    //   x Conclude

    public DeleTransaction(PopConnection connection, PopRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Parameters.ContainsKey("Message-Number"))
        return ProcessParameterNotSetted;

      messageNumber = Request.Parameters["Message-Number"];

      return ProcessDele;
    }

    private void ProcessParameterNotSetted()
    {
      Conclude(new PopResponse(PopTransactionStatus.BadRequest, "parameter 'Message-Number' must be setted"));
    }

    private void ProcessDele()
    {
      Send("DELE", messageNumber);

      Transit(WaitForDele);
    }

    private void WaitForDele()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopStatusResponse) {
        var status = response as PopStatusResponse;

        if (status.Status == PopProtocolStatus.Positive) {
          Conclude(new PopResponse(PopTransactionStatus.Ok, string.Format("message-number {0} is marked to be deleted", messageNumber)));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string messageNumber;
  }
}
