using System;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Transaction.Transactions {
  internal class GreetingTransaction : TransactionBase {

    // WaitForGreeting
    //   x Conclude

    public GreetingTransaction(PopConnection connection, PopRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      Trace.Verbose("[POP3] waiting for server greeting response");

      return WaitForGreeting;
    }

    private void WaitForGreeting()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopStatusResponse) {
        var status = response as PopStatusResponse;

        if (status.Status == PopProtocolStatus.Positive) {
          var timestamp = string.Empty;
          if (GetTimestamp(status.Message, out timestamp)) {
            Connection.ServerInfo.Timestamp = timestamp;
            Connection.ServerInfo.Capability.KnownCapabilities |= PopKnownCapabilities.Apop;
            Conclude(new PopResponse(PopTransactionStatus.Ok, string.Format("greeted (APOP is available, timestamp is '{0}')", timestamp)));
          }
          else {
            Conclude(new PopResponse(PopTransactionStatus.Ok, "greeted"));
          }
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private bool GetTimestamp(string message, out string timestamp)
    {
      timestamp = string.Empty;

      var start = message.IndexOf('<');

      if (start == -1)
        return false;

      var end = message.IndexOf('>', start);

      if (end == -1)
        return false;

      timestamp = message.Substring(start, end - start + 1);

      return true;
    }
  }
}
