using System;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Transaction.Transactions {
  internal class StatTransaction : TransactionBase {

    // ProcessStat
    //   ->WaitForStat
    // WaitForStat
    //   x Conclude

    public StatTransaction(PopConnection connection, PopRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessStat;
    }

    private void ProcessStat()
    {
      Send("STAT");

      Transit(WaitForStat);
    }

    private void WaitForStat()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopStatusResponse) {
        var status = response as PopStatusResponse;

        if (status.Status == PopProtocolStatus.Positive) {
          PopDropList dropList;

          if (RetrieveDropList(status, out dropList))
            Conclude(new PopDropListResponse(PopTransactionStatus.Ok, dropList, "retrieved drop lists"));
          else
            Conclude(new PopResponse(PopTransactionStatus.BadRequest, "invalid drop list format"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private bool RetrieveDropList(PopStatusResponse status, out PopDropList dropList)
    {
      dropList = null;

      var resp = status.Message.Split(new[] {' '}, StringSplitOptions.None);

      if (resp.Length < 2)
        return false;

      int messages;

      if (!int.TryParse(resp[0], out messages))
        return false;

      int octets;

      if (!int.TryParse(resp[1], out octets))
        return false;

      dropList = new PopDropList(messages, octets);

      return true;
    }
  }
}
